﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace Agenda_XML_Cs
{
    public partial class DataGrid : Form
    {
        public DataGrid()
        {
            InitializeComponent();
        }

        private DataSet datos;
        private string ruta;

        // al arrancar el programa, cargar el origen de datos XML en DataGridView
        private void DataGrid_Load(object sender, EventArgs e)
        {
            // botón Cancelar por defecto, al pulsar la tecla ESC se sale de la aplicación
            this.CancelButton = this.BtSalir;
            // cargar el archivo XML
            try
            {
                // ruta al archivo XML, en la carpeta del ejecutable
                ruta = Path.Combine(Application.StartupPath, "500empresas.xml");
                // DataSet: contiene una copia de los datos en esquema XML, independiente del proveedor, 
                // con sus elementos, tablas y relaciones, es el verdadero almacén de datos desconectados
                datos = new DataSet();
                // Rellenar el DataSet
                datos.ReadXml(ruta);
                // asignar Dataset  y tabla a DataGridView
                this.Grid.DataSource = this.datos;
                this.Grid.DataMember = "AgendaTb";
                //
                // ancho de las columnas del DataGridView para que se ajusten al contenido de las celdas incluyendo las cabeceras
                this.Grid.AutoResizeColumns();
                this.Grid.AutoSizeColumnsMode = DataGridViewAutoSizeColumnsMode.Fill;
                // el ancho de la primera columna se configura más abajo por código después de crear DataView
                //
                // Para diseñar estilos de cabeceras por código hay que modificar la propiedad DataGridView.EnableHeadersVisualStyles
                // que por defecto tiene valor true, con lo que las cabeceras del control utilizarán la combinación de colores del tema seleccionado 
                // en el sistema operativo, hay que poner esa propiedad a false para que las cabeceras tengan el diseño del estilo
                //
                // creación de estilo para las celdas de DataGridView
                DataGridViewCellStyle estilo1;
                estilo1 = new DataGridViewCellStyle
                {
                    BackColor = Color.White,
                    ForeColor = Color.Black,
                    Alignment = DataGridViewContentAlignment.MiddleLeft,
                    NullValue = "",
                    Font = new Font("Calibri", 10, FontStyle.Regular),
                    SelectionBackColor = Color.Gainsboro,
                    SelectionForeColor = Color.Black
                };
                // asignar estilo a las celdas
                this.Grid.DefaultCellStyle = estilo1;
                //
                // creación de estilo para las cabeceras de DataGridView 
                DataGridViewCellStyle estilo2;
                estilo2 = new DataGridViewCellStyle
                {
                    BackColor = Color.Black,
                    ForeColor = Color.Yellow,
                    Alignment = DataGridViewContentAlignment.MiddleCenter,
                    Font = new Font("Calibri", 10, FontStyle.Regular)
                };
                // asignar estilo a las cabeceras
                this.Grid.ColumnHeadersDefaultCellStyle = estilo2;
                //
                // creación de estilo para filas alternas de DataGridView 
                DataGridViewCellStyle estilo3;
                estilo3 = new DataGridViewCellStyle
                {
                    BackColor = Color.LightYellow,
                    Alignment = DataGridViewContentAlignment.MiddleLeft,
                    Font = new Font("Calibri", 10, FontStyle.Regular)
                };
                // asignar estilo a las filas alternas 
                this.Grid.AlternatingRowsDefaultCellStyle = estilo3;
                //
                // abrir la agenda por la letra A (registros ordenados)
                //this.TxFiltro.Text = "A";
                // filtro SQL: registros que empiecen por el texto escrito en el TextBox, para ello se utliza el objeto DataView
                // que es una vista personalizada conectada de una DataTable apta para ordenar, filtrar, navegar, etc.
                DataView vista = new DataView
                {
                    Table = datos.Tables["Agendatb"],
                    RowFilter = ("Nombre LIKE \'"
                            + (this.TxFiltro.Text + "%\'")),
                    Sort = "Nombre"
                };
                // nuevo origen del DataDrid: la vista personalizada
                this.Grid.DataSource = vista;
                //
                // configurar por código el tamaño de la primera columna de DataGridView
                this.Grid.Columns[0].Width = 320;
                // configurar por código el nº máximo de caracteres que aceptan las columnas de los teléfonos y de ID
                ((DataGridViewTextBoxColumn)Grid.Columns[1]).MaxInputLength = 9;
                ((DataGridViewTextBoxColumn)Grid.Columns[2]).MaxInputLength = 9;
                // cambiar el tipo de dato de las columnas de teléfonos a int32
                //MessageBox.Show("Columns[1].ValueType " + this.Grid.Columns[1].ValueType.ToString());
                //MessageBox.Show("Columns[2].ValueType " + this.Grid.Columns[2].ValueType.ToString());
                this.Grid.Columns[1].ValueType = typeof(Int32);
                this.Grid.Columns[2].ValueType = typeof(Int32);
                //MessageBox.Show("Columns[1].ValueType " + this.Grid.Columns[1].ValueType.ToString());
                //MessageBox.Show("Columns[2].ValueType " + this.Grid.Columns[2].ValueType.ToString());
                //
                // contar el número de registros que presenta el DataGrid
                // se pueden contar las filas visibles del DataGrid o los registros mostrados por DataView
                int n;
                //n = Grid.Rows.GetRowCount(DataGridViewElementStates.Visible);
                n = vista.Count;
                LbRegistros.Text = ("Número de registros: " + n.ToString());
                this.Grid.Update();
                // sin celda seleccionada al arrancar DataGridView
                //this.Grid.CurrentCell = null;
                // Habilitar botones Actualizar y Quitar filtro y permitir escribir en TextBox
                this.BtActualizar.Enabled = true;
                this.BtQuitar.Enabled = true;
                // Habilitar la escritura en el Texbox
                this.TxFiltro.ReadOnly = false;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error en private void BtCargar_Click", ex.Message);
            }
        }

        //private void Grid_SortCompare(object sender, System.Windows.Forms.DataGridViewSortCompareEventArgs e)
        //{
        //    if (e.Column.Index == 3) {
        //        e.SortResult = int.Parse(e.CellValue1.ToString()).CompareTo(int.Parse(e.CellValue2.ToString()));
        //        e.Handled = true;
        //    }
        //}

        // lo que sucede al pulsar cualquiera de los botones de las letras del alfabeto
        private void BtA_Click(object sender, EventArgs e)
        {
            try
            {
                // obtener la letra pulsada (texto del botón)
                string letra = (sender as Button).Text;
                // ponerla en mayúsculas
                letra = letra.ToUpper();
                // filtro SQL: registros que empiecen por el texto del botón
                DataView vista = new DataView
                {
                    Table = datos.Tables["Agendatb"],
                    RowFilter = ("Nombre LIKE \'" + (letra + "%\'")),
                    Sort = "Nombre"
                };
                // nuevo origen del DataDrid: la vista personalizada
                this.Grid.DataSource = vista;
                // recargar el DataGrid
                this.Grid.Update();
                // contar el número de registros que presenta el DataGrid
                // se pueden contar las filas visibles del DataGrid o los registros mostrados por DataView
                int n;
                //n = Grid.Rows.GetRowCount(DataGridViewElementStates.Visible);
                n = vista.Count;
                this.LbRegistros.Text = ("Número de registros: " + n.ToString());
                this.TxFiltro.Text = letra;
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error en BtA_Click", ex.Message);
            }
        }

        // el botón Filtrar selecciona registros según lo escrito en el cuadro de texto
        // y busca coinicidencias al principio del campo Nombre
        private void BtFiltrar1_Click(object sender, EventArgs e)
        {
            // aplicar un filtro a la tabla AgendaTb del DataSet mediante un DataView
            // DataView permite personalizar las vistas de las tablas
            try
            {
                // si hay algo escrito en el cuadro de texto
                if ((this.TxFiltro.Text.Length > 0))
                {
                    // filtro SQL: registros que empiecen por el texto escrito en el TextBox
                    DataView vista = new DataView
                    {
                        Table = datos.Tables["Agendatb"],
                        RowFilter = ("Nombre LIKE \'" + (this.TxFiltro.Text + "%\'")),
                        Sort = "Nombre"
                    };
                    // nuevo origen del DataDrid: la vista personalizada
                    this.Grid.DataSource = vista;
                    // recargar el DataGrid
                    this.Grid.Update();
                    // contar el número de registros que presenta el DataGrid
                    // se pueden contar las filas visibles del DataGrid o los registros mostrados por DataView
                    int n;
                    //n = Grid.Rows.GetRowCount(DataGridViewElementStates.Visible);
                    n = vista.Count;
                    this.LbRegistros.Text = ("Número de registros: " + n.ToString());
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error en BtFiltrar_Click", ex.Message);
            }
        }

        // el botón Filtrar selecciona registros según lo escrito en el cuadro de texto
        // y busca coinicidencias en cualquier lugar del campo Nombre
        private void BtFiltrar2_Click(object sender, EventArgs e)
        {
            try
            {
                // si hay algo escrito en el cuadro de texto
                if ((this.TxFiltro.Text.Length > 0))
                {
                    // filtro SQL: registros que contengan el texto escrito en el TextBox
                    DataView vista = new DataView
                    {
                        Table = datos.Tables["Agendatb"],
                        RowFilter = ("Nombre LIKE \'%" + (this.TxFiltro.Text + "%\'")),
                        Sort = "Nombre"
                    };
                    // nuevo origen del DataDrid: la vista personalizada
                    this.Grid.DataSource = vista;
                    // recargar el DataGrid
                    this.Grid.Update();
                    //
                    // contar el número de registros que presenta el DataGrid
                    // se pueden contar las filas visibles del DataGrid o los registros mostrados por DataView
                    int n;
                    //n = Grid.Rows.GetRowCount(DataGridViewElementStates.Visible);
                    n = vista.Count;
                    this.LbRegistros.Text = ("Número de registros: " + n.ToString());
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error en BtFiltrar_Click", ex.Message);
            }
        }

        // volver a mostrar todos los registros
        private void BtQuitar_Click(object sender, EventArgs e)
        {
            try
            {
                DataView vista = new DataView
                {
                    Table = datos.Tables["Agendatb"],
                    Sort = "Nombre"
                };
                // recuperar de nuevo todos los registros
                this.Grid.DataSource = vista;
                // recargar el DataGrid
                this.Grid.Update();
                //
                int n;
                //n = Grid.Rows.GetRowCount(DataGridViewElementStates.Visible);
                n = vista.Count;
                this.LbRegistros.Text = ("Número de registros: " + n.ToString());
                this.TxFiltro.Text = "";
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error en BtQuitar_Click", ex.Message);
            }
        }

        // actualizar archivo XML con los cambios realizados en DataGridView
        private void BtActualizar_Click(object sender, EventArgs e)
        {
            try
            {
                this.Grid.EndEdit();
                //if (!datos.HasChanges()) return;
                //bool cambios;
                //cambios = datos.HasChanges();
                //MessageBox.Show(cambios.ToString());
                //if (cambios == true) {
                // escribir el DataSet al archivo XML
                datos.WriteXml(ruta);
                datos.Clear();
                datos.ReadXml(ruta);
                //string aviso5 = "Archivo XML modificado con éxito";
                //MessageBox.Show(aviso5, "Información");
                int n;
                n = this.Grid.Rows.GetRowCount(DataGridViewElementStates.Visible) - 1;
                LbRegistros.Text = ("Número de registros: " + n.ToString());
                this.TxFiltro.Text = "";
                //}
            }
            catch (Exception ex)
            {
                MessageBox.Show("Error en BtActualizar_Click", ex.Message);
            }
        }

        // salir del programa
        private void BtSalir_Click(object sender, EventArgs e)
        {
            this.Close();
        }

    }
}
